program checker (output);

{
  Checker voor Feest 2000 opgave
  TU Eindhoven
  Aanpassingen door Joris
}

const
  InFileName = 'feest.in';
  OutFileName = 'feest.uit';
  LogFileName = 'feest.log';

  { party.inp: original input file (read)
    party.out: output file to be checked (read)
    party.log: log file with messages (write)
    output (=stdout): error messages

    N.B. stdout remains empty iff party.out is correct w.r.t. party.inp
  }

const
  MaxR = 100;
  MaxN = 100;

{ type
  string = varying [80] of char; for Sun Pascal }

var
  inp, out, logfile: text;
  r: integer; { # runs (read from inp) }
  i: integer; { traverses runs }
  n: integer; { # guests (read from inp) }
  m: integer; { minimal # rounds possible for n guests }
  k: integer; { # rounds (read from out) }
  j: integer; { traverses rounds }
  line: integer; { # lines read so far from out }
  pairs: array [ 0..MaxN-1, 0..MaxN-1 ] of integer;
    { pairs [ i, j ] = # of round in which i shakes hands with j, 0 if none }
  shakes: integer; { # handshakes done so far }
  busy: array [ 0..MaxN-1 ] of boolean;
    { busy[i] = i is shaking hands in current round }
  g, h: integer; { guests (read from out) }
  c: char; { dummy to read '-' }
  pendingchar: char;
const
  errors: integer = 0;

procedure writemessage(s: string);
  begin
    inc(errors);
    writeln(output, s, ': line=', line+1:1, ', run=', i:1, ', round=', j:1)
  ; writeln(logfile, s, ': line=', line+1:1, ', run=', i:1, ', round=', j:1)
  end; { writemessage }

function CharToDigit(c: char): integer;
  begin
    CharToDigit := ord(c) - ord('0')
  end; { CharToDigit }

procedure readint(var f: text; var v: integer; w: integer);
  { read unsigned integer from f; error if more than w digits }
  const Digits = ['0'..'9']; Radix = 10;
  var c: char; i: integer;
  begin
    v := 0 ; i := 0 { i = # digits read from f }
  ; pendingchar := ' ';
  ; while not eoln(f) do begin
      read(f, c);
      if not (c in Digits) then begin pendingchar := c; break; end
    ; v := v*Radix + CharToDigit(c)
    ; i := succ(i)
    end { while }
  ; if not(i<>0) then writemessage('Number expected')
    else if not(i<=w) then writemessage('Too many digits in number')
  end; { readint }


procedure checkeoln(var f: text);
  begin
    if eoln(f) then writemessage('Premature EOLN')
  end; { checkeof }

procedure checkeof(var f: text);
  begin
    if eof(f) then writemessage('Premature EOF')
  end; { checkeof }

procedure initpairs;
  var i, j: integer;
  begin
    for i := 0 to n-1 do
      for j := 0 to n-1 do
        pairs[i, j] := 0
  end; { initpairs }

procedure initbusy;
  var i: integer;
  begin
    for i := 0 to n-1 do
      busy[i] := false
  end; { initp }

begin { checker }

  assign(inp, InFileName)
; reset(inp)
  { N.B. file inp is assumed correctly formatted,
    nevertheless some errors are flagged }
; line := 0
; assign(out, OutFileName)
; reset(out)
  { N.B. file out is suspect in every possible way }
; assign(logfile, LogFileName)
; rewrite(logfile)

; readln(inp, r)
; writeln(logfile, '# runs = ', r:1)
; if not ((0<=r) and (r<=MaxR)) then
    writemessage('INPUT ERROR: # runs out of range')

; for i := 1 to r do begin

    readln(inp, n)
  ; writeln(logfile, '# guests = ', n:1)
  ; if not ((1<=n) and (n<=MaxN)) then
      writemessage('INPUT ERROR: # guests out of range')

  ; if (n<>1) and odd(n) then m := n
    else m := n - 1
  ; initpairs ; shakes := 0
  ; checkeoln(out)
  ; readint(out, k, 3)
  ; writeln(logfile, '# rounds = ', k:1)
  ; if not (0<=k) then writemessage('# rounds out of range')
  ; if not(k=m) then writemessage('# rounds not minimal')
  ; if (not eoln(out)) or (pendingchar <> ' ') then writemessage('Superfluous output at end of line')
  ; readln(out) ; line := line + 1

  ; for j := 1 to k do begin { check round j }

      initbusy

    ; while not eoln(out) do begin
      ; checkeoln(out)
      ; readint(out, g, 3)
      ; write(logfile, g:1)
      ; if not ((0<=g) and (g<n)) then writemessage('Left guest out of range')
      ; if busy[g] then writemessage('Shaking hands simultaneously')
      ; busy[g] := true
{      ; checkeoln(out) }
      ; c := pendingchar;
{      ; read(out, c) }
      ; write(logfile, c)
      ; if c <> '-' then writemessage('Incorrect connector')
      ; checkeoln(out)
      ; readint(out, h, 3)
      ; write(logfile, h:1, ' ')
      ; if not ((0<=h) and (h<n)) then writemessage('Right guest out of range')
      ; if g=h then writemessage('Shaking one''s own hand')
        else if busy[h] then writemessage('Shaking hands simultaneously')
      ; busy[h] := true
      ; if pairs[g, h] <> 0 then writemessage('Duplicate handshake')
      ; shakes := shakes + ord((g<>h) and (pairs[g, h] = 0))
      ; pairs[g, h] := j ; pairs[h, g] := j
(*      ; if not eoln(out) then begin
          read(out, c) *)
        ; if pendingchar <> ' ' then writemessage('Incorrect separator')
(*        end { if } *)
      end { while }

    ; writeln(logfile)
    ; readln(out) ; line := line + 1

    end { for j }

  ; if shakes <> (n*(n-1)) div 2 then writemessage('not all pairs shook hands')

  end { for i }

; if not eof(out) then writemessage('Too much output')
; close(logfile)
; if errors > 0 then halt(1);
end.
